﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Encodings.DecisionList.Mutation {
  [Item("UniformOnePointMutation", "Description missing")]
  [StorableClass]
  public class UniformOnePointManipulator : DecisionListManipulator {

    [StorableConstructor]
    protected UniformOnePointManipulator(bool deserializing) : base(deserializing) { }
    protected UniformOnePointManipulator(UniformOnePointManipulator original, Cloner cloner)
      : base(original, cloner) {
    }
    public UniformOnePointManipulator()
      : base() {
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new UniformOnePointManipulator(this, cloner);
    }

    public static void Apply(IRandom random, DecisionList child, double actionMutationProbability) {
      int rulePos = random.Next(0, child.Rules.Count);
      Rule mutateRule = child.Rules[rulePos];

      var possiblePosition = new List<int>() { mutateRule.Action.CurrentPosition };
      if (child.DefaultAction != null) { possiblePosition.Add(child.DefaultAction.CurrentPosition); }
      var remainingPositions = mutateRule.Action.GetPossibleActionPositions().Except(possiblePosition);

      if (remainingPositions.Count() > 0 && random.NextDouble() < actionMutationProbability) {
        int pos = random.Next(0, remainingPositions.Count());
        mutateRule.Action.SetToPosition(remainingPositions.ElementAt(pos));
      } else {
        int attributePos = random.Next(0, mutateRule.Variables.Count);
        string variableName = mutateRule.Variables.Keys.ToList()[attributePos];
        mutateRule.Variables[variableName].Mutate(random);
      }
    }

    protected override void Manipulate(IRandom random, DecisionList child, double actionMutationProbability) {
      Apply(random, child, actionMutationProbability);
    }
  }
}
